#pragma once
#include <iostream>
#include <sstream>
#include <boost/uuid/uuid.hpp>
#include <boost/uuid/uuid_generators.hpp>
#include <boost/uuid/uuid_io.hpp>
#include "domain/physical_quantities.h"

namespace spin
{
    using UUID = boost::uuids::uuid;

    struct Program
    {

        Program(const boost::uuids::uuid &uuid,
                const std::string &name,
                const Length &reference_depth,
                const Length &start_depth,
                const Length &initial_safety_shield_position,
                const bool is_force_controlled) : uuid_(uuid),
                                                  name_(name),
                                                  reference_depth_(reference_depth),
                                                  start_depth_(start_depth),
                                                  initial_safety_shield_position_(initial_safety_shield_position),
                                                  is_force_controlled_(is_force_controlled) {}

        inline friend std::ostream &operator<<(std::ostream &out, const Program &program)
        {
            out << "uuid: " << program.uuid_ << std::endl;
            out << "name: " << program.name_ << std::endl;
            out << "reference_depth: " << program.reference_depth_.toMillimeters() << " mm" << std::endl;
            out << "start_depth: " << program.start_depth_.toMillimeters() << " mm" << std::endl;
            out << "initial_safety_shield_position: " << program.initial_safety_shield_position_.toMillimeters() << " mm" << std::endl;
            out << "is_force_controlled: " << program.is_force_controlled_ << std::endl;
            return out;
        }

        std::string toString() const
        {
            std::ostringstream oss;
            oss << *this;
            return oss.str();
        }

        inline bool operator<(const Program &rhs) const
        {
            return name_ < rhs.name_;
        }

        boost::uuids::uuid uuid_;
        std::string name_;
        spin::Length reference_depth_;
        Length start_depth_;
        Length initial_safety_shield_position_;
        bool is_force_controlled_;
    };
} // namespace spin